@@ -2,18 +2,18 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import print_function
+import os
+import sys
 import argparse
+import logging
+import subprocess
 import httplib
 import urllib
 import random
 import string
 import mimetypes
-import sys
 from ConfigParser import SafeConfigParser
 import json
-import logging
-import os
-import subprocess
 from abc import ABCMeta
 from abc import abstractmethod
 import time
@@ -21,7 +21,6 @@ from functools import wraps
 import math
 import shutil
 
-logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.INFO)
 logger = logging.getLogger(__name__)
 
 
@@ -94,7 +93,7 @@ class Imgur():
                           'you have to visit this URL in your browser '
                           'and copy the PIN code: \n')
         self._auth_msg_with_url = self._auth_msg + self._auth_url
-        self._token_msg = 'Enter PIN code displayed in the browser: '
+        self._enter_token_msg = 'Enter PIN code displayed in the browser: '
         self._no_album_msg = 'Do not move to any album'
 
     def retry(tries=2, delay=1):
@@ -163,7 +162,11 @@ class Imgur():
         '''
         args = self.get_error_dialog_args(msg)
         if args:
-            p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            p = subprocess.Popen(
+                args,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE
+            )
             p.communicate()
         logger.error(msg)
         sys.exit(1)
@@ -296,15 +299,15 @@ class Imgur():
         '''
         pass
 
-    def ask_pin(self):
+    def ask_pin(self, auth_msg, auth_url, enter_token_msg):
         '''
         Ask user for pin code
         Returns:
             pin code
         '''
         args = self.get_auth_msg_dialog_args(
-            self._auth_msg,
-            self._auth_url,
+            auth_msg,
+            auth_url
         )
         auth_msg_dialog = subprocess.Popen(
             args,
@@ -313,7 +316,7 @@ class Imgur():
         )
         auth_msg_dialog.communicate()
 
-        args = self.get_enter_pin_dialog_args(self._token_msg)
+        args = self.get_enter_pin_dialog_args(enter_token_msg)
         ask_pin_dialog = subprocess.Popen(
             args,
             stdout=subprocess.PIPE,
@@ -328,8 +331,15 @@ class Imgur():
         '''
         token_url = '/oauth2/token'
 
-        pin = self.ask_pin()
-        headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
+        pin = self.ask_pin(
+            self._auth_msg,
+            self._auth_url,
+            self._enter_token_msg
+        )
+        headers = {
+            "Content-type": "application/x-www-form-urlencoded",
+            "Accept": "text/plain"
+        }
         self._request(
             'POST',
             token_url,
@@ -456,18 +466,19 @@ class Imgur():
         '''
         pass
 
-    def show_link(self, result):
+    def show_link(self, image_link, delete_hash):
         '''
         Show image link
         Args:
             result: Image upload response(json(dict))
         Returns:
-            list: A list include dialog command, ex: ['kdialog', '--msgbox', 'hi']
+            list: A list include dialog command,
+                  ex: ['kdialog', '--msgbox', 'hi']
         '''
-        link = 'Link: {link}'.format(link=result['data']['link'].replace('\\', ''))
+        link = 'Link: {link}'.format(link=image_link.replace('\\', ''))
         links = (
             link + '\nDelete link: http://imgur.com/delete/' +
-            '{delete}'.format(delete=result['data']['deletehash'])
+            '{delete}'.format(delete=delete_hash)
         )
         args = self.get_show_link_dialog_args(links)
         show_link_dialog = subprocess.Popen(
@@ -528,7 +539,7 @@ class Imgur():
         self._request('POST', url, body, headers)
         return self._get_json_response()
 
-    def upload(self, image_path=None, anonymous=True, album_id=None):
+    def upload(self, image_path=None, meta=None):
         '''
         Upload a image
         Args:
@@ -541,7 +552,7 @@ class Imgur():
         headers = {}
         if image_path is None:
             image_path = self.ask_image_path()
-        if anonymous:  # Anonymous account
+        if meta['anonymous']:  # Anonymous account
             print('Upload the image anonymously...')
             files = {'image': image_path}
             body, headers = self._encode_multipart_data(data, files)
@@ -553,34 +564,25 @@ class Imgur():
                 # tool, so call auth() to get tokens
                 self.auth()
                 self.write_tokens_to_config()
-            if album_id is None:  # Means user doesn't specify the album
+            if (meta['album_id'] is None) or meta['ask']:  # Means user doesn't specify the album
                 albums = self.request_album_list()
-                album_id = self.ask_album_id(albums)
-                if album_id is not None:
+                meta['album_id'] = self.ask_album_id(albums)
+                if meta['album_id'] is not None:
                     logger.info('Upload the image to the album...')
-                    data['album_id'] = album_id
+                    data['album_id'] = meta['album_id']
                 else:
                     # If it's None, means user doesn't want to upload to any album
                     logger.info('Upload the image...')
             else:
                 logger.info('Upload the image to the album...')
-                data['album_id'] = album_id
+                data['album_id'] = meta['album_id']
 
             files = {'image': image_path}
             body, headers = self._encode_multipart_data(data, files)
             headers['Authorization'] = 'Bearer {access_token}'.format(access_token=self._access_token)
 
-        self._request('POST', url, body, headers)
-        result = self._get_json_response()
-        if not self.is_success(result):
-            logger.info('Reauthorize...')
-            self.request_new_tokens_and_update()
-            self.write_tokens_to_config()
-            self._request('POST', url, body, headers)
-            result = self._get_json_response()
-            if not self.is_success(result):
-                self.show_error_and_exit('Upload image error')
-        self.show_link(result)
+        result = self.request_upload_image(url, body, headers)
+        self.show_link(result['link'], result['deletehash'])
 
 
 class CLIImgur(Imgur):
@@ -594,9 +596,9 @@ class CLIImgur(Imgur):
     def get_enter_pin_dialog_args(self, token_msg):
         pass
 
-    def ask_pin(self):
-        print(self._auth_msg_with_url)
-        pin = raw_input(self._token_msg)
+    def ask_pin(self, auth_msg, auth_url, enter_token_msg):
+        print(auth_msg + auth_url)
+        pin = raw_input(enter_token_msg)
         return pin
 
     def get_ask_image_path_dialog_args(self):
@@ -626,9 +628,9 @@ class CLIImgur(Imgur):
     def get_show_link_dialog_args(self):
         pass
 
-    def show_link(self, result):
-        print('Link: {link}'.format(link=result['data']['link'].replace('\\', '')))
-        print('Delete link: http://imgur.com/delete/{delete}'.format(delete=result['data']['deletehash']))
+    def show_link(self, image_link, delete_hash):
+        print('Link: {link}'.format(link=image_link.replace('\\', '')))
+        print('Delete link: http://imgur.com/delete/{delete}'.format(delete=delete_hash))
 
 
 class KDEImgur(Imgur):
@@ -730,7 +732,10 @@ class MacImgur(Imgur):
         args = [
             'osascript',
             '-e',
-            'tell app "Finder" to POSIX path of (choose file with prompt "Choose Image:")',
+            (
+                'tell app "Finder" to POSIX path of '
+                '(choose file with prompt "Choose Image:")'
+            ),
         ]
         return args
 
@@ -749,8 +754,8 @@ class MacImgur(Imgur):
             'osascript',
             '-e',
             (
-                'tell app "Finder" to choose from list '
-                '{{{l}}} with title "Choose From The List" with prompt "PickOne" '
+                'tell app "Finder" to choose from list {{{l}}} '
+                'with title "Choose From The List" with prompt "PickOne" '
                 'OK button name "Select" cancel button name "Quit"'.format(l=list_str[:-1])
             ),
         ]
@@ -769,8 +774,8 @@ class MacImgur(Imgur):
     def get_show_link_dialog_args(self):
         pass
 
-    def show_link(self, result):
-        link = result['data']['link'].replace('\\', '')
+    def show_link(self, image_link, delete_hash):
+        link = image_link.replace('\\', '')
         args = [
             'osascript',
             '-e',
@@ -790,7 +795,7 @@ class MacImgur(Imgur):
         response = response[response.find(':') + 1:response.find(',')]
         print(response)
         if response == 'Show delete link':
-            delete_link = 'http://imgur.com/delete/{delete}'.format(delete=result['data']['deletehash'])
+            delete_link = 'http://imgur.com/delete/{delete}'.format(delete=delete_hash)
             args2 = [
                 'osascript',
                 '-e',
@@ -870,10 +875,19 @@ class ZenityImgur(Imgur):
 
 
 def main():
+    formatter = logging.Formatter('%(levelname)s: %(message)s')
+    console = logging.StreamHandler(stream=sys.stdout)
+    console.setLevel(logging.DEBUG)
+    console.setFormatter(formatter)
+    logger.addHandler(console)
+    logger.setLevel(logging.INFO)
+
     parser = argparse.ArgumentParser()
     parser.add_argument(
         '-f',
-        help='The image you want to upload',
+        nargs='*',
+        default=None,
+        help='The images you want to upload',
         metavar='<image path>'
     )
     parser.add_argument(
@@ -883,6 +897,11 @@ def main():
         help='The album id you want your image to be uploaded to',
         metavar='<album id>'
     )
+    parser.add_argument(
+        '-q',
+        action='store_true',
+        help='Choose album with each file'
+    )
     parser.add_argument(
         '-g',
         action='store_true',
@@ -897,7 +916,7 @@ def main():
         '-s',
         action='store_true',
         help='Add command in the context menu of file manager'
-        '(Now only support KDE)'
+        '(Support KDE and Gnome)'
     )
     args = parser.parse_args()
 
@@ -906,7 +925,14 @@ def main():
                      os.path.expanduser('~/.local/share/applications/'))
         return
     imgur = ImgurFactory.get_imgur(ImgurFactory.detect_env(args.g))
-    imgur.upload(args.f, args.n, args.d)
+
+    meta = {
+        'album_id': args.d,
+        'ask': args.q,
+        'anonymous': args.n,
+    }
+    for f in args.f:
+        imgur.upload(f, meta)
 
 
 if __name__ == '__main__':
